@@ -1,6 +1,9 @@
+import numpy as np
+
 from guide_bot.base_elements import guide_elements
 from guide_bot.parameters import instrument_parameters as ipars
 
+from guide_bot.base_elements.guide_elements import PositionAndRotation
 
 class Elliptic(guide_elements.GuideElement):
     """
@@ -71,50 +74,6 @@ class Elliptic(guide_elements.GuideElement):
         max_height : float
             Maximum height of the inserted guide segment
         """
-
-        """
-        # Internalize stuff relevant for this module
-        self.minor_axis_x = guide_elements.handle_input_parameter(name + "_minor_axis_x", minor_axis_x,
-                                                                  default_min=-0.3, default_max=0.3)
-
-        self.minor_axis_y = guide_elements.handle_input_parameter(name + "_minor_axis_y", minor_axis_y,
-                                                                  default_min=-0.3, default_max=0.3)
-
-        # max_width is the upper limit for minor axis, which is not true, it should be upper limit
-        #  for the actual width of the guide segment which can be much smaller for e.g. a parabola
-
-        self.max_width = max_width
-        # Set limits for start/end width to a maximum of max width
-        if self.max_width is not None:
-            start_width = guide_elements.handle_input_parameter(name + "_start_width", start_width,
-                                                                default_min=0.005, default_max=self.max_width)
-            if isinstance(start_width, ipars.FreeInstrumentParameter):
-                if start_width.get_upper_bound() > self.max_width:
-                    start_width.upper_bound = self.max_width
-
-            if end_width is not None:
-                end_width = guide_elements.handle_input_parameter(name + "_end_width", end_width,
-                                                                  default_min=0.005, default_max=self.max_width)
-                if isinstance(end_width, ipars.FreeInstrumentParameter):
-                    if end_width.get_upper_bound() > self.max_width:
-                        end_width.upper_bound = self.max_width
-
-        self.max_height = max_height
-        # Set limits for start/end height to a maximum of max height
-        if self.max_height is not None:
-            start_height = guide_elements.handle_input_parameter(name + "_start_height", start_height,
-                                                                 default_min=0.005, default_max=self.max_height)
-            if isinstance(start_height, ipars.FreeInstrumentParameter):
-                if start_height.get_upper_bound() > self.max_height:
-                    start_height.upper_bound = self.max_height
-
-            if end_height is not None:
-                end_height = guide_elements.handle_input_parameter(name + "_end_height", end_height,
-                                                                   default_min=0.005, default_max=self.max_height)
-                if isinstance(end_height, ipars.FreeInstrumentParameter):
-                    if end_height.get_upper_bound() > self.max_height:
-                        end_height.upper_bound = self.max_height
-        """
         # Record if the parameters are given by the user or not
         self.start_width_custom_parameter = isinstance(start_width, ipars.InstrumentParameter)
         self.minor_axis_x_custom_parameter = isinstance(minor_axis_x, ipars.InstrumentParameter)
@@ -293,34 +252,6 @@ class Elliptic(guide_elements.GuideElement):
         self.current_parameters.add_parameter(self.minor_axis_x)
         self.current_parameters.add_parameter(self.minor_axis_y)
 
-        """
-        # Add constraints ensuring that minor axis is wider than both start and end
-        # If the user has set both minor_axis and a dimension with the same userdefined parameter, they will
-        #  be equal, and thus the constraint is not needed. Furthermore, it would be a floating point rounding
-        #  that determines whether the constraint is fulfilled or not, causing problems.
-        if self.minor_axis_x is not self.start_width:
-            self.current_parameters.add_new_constraint([self.minor_axis_x, self.start_width], lambda x, y: abs(x) - y)
-        if self.minor_axis_x is not self.end_width:
-            self.current_parameters.add_new_constraint([self.minor_axis_x, self.end_width], lambda x, y: abs(x) - y)
-
-        if self.minor_axis_y is not self.start_height:
-            self.current_parameters.add_new_constraint([self.minor_axis_y, self.start_height], lambda x, y: abs(x) - y)
-        if self.minor_axis_y is not self.end_height:
-            self.current_parameters.add_new_constraint([self.minor_axis_y, self.end_height], lambda x, y: abs(x) - y)
-
-        # When minor_axis_x is negative, the minor axis is outside, and the widest point is the largest entry/exit
-        # When minor_axis_y is positive, the minor axis is inside, and the widest point is the small axis
-        if self.max_width is not None:
-            self.current_parameters.add_new_constraint(self.start_width, lambda x, a: a - x, constants=self.max_width)
-            self.current_parameters.add_new_constraint(self.end_width, lambda x, a: a - x, constants=self.max_width)
-            self.current_parameters.add_new_constraint(self.minor_axis_x, lambda x, a: a - x, constants=self.max_width)
-
-        if self.max_height is not None:
-            self.current_parameters.add_new_constraint(self.start_height, lambda x, a: a - x, constants=self.max_height)
-            self.current_parameters.add_new_constraint(self.end_height, lambda x, a: a - x, constants=self.max_height)
-            self.current_parameters.add_new_constraint(self.minor_axis_y, lambda x, a: a - x, constants=self.max_height)
-        """
-
         # Instruct instrument to calculate length and get name
         length_name = self.get_length_name()
 
@@ -396,10 +327,10 @@ class Elliptic(guide_elements.GuideElement):
         instr.append_initialize(Louty_par_name + '=' + Liny_par_name + '+tmp_L1-tmp_L2;')
 
         # Add the guide component and use the calculated variables for its properties
-        guide = self.current_instrument.add_component(self.name, "Elliptic_guide_gravity", after="Origin")
-        guide.set_AT([0, 0, self.start_point_parameter.name])
+        guide = self.current_instrument.add_component(self.name, "Elliptic_guide_gravity")
+        guide.set_AT([0, 0, 0], RELATIVE=self.reference_component_name)
 
-        guide.l = length_name
+        guide.l = length_name + " - 1E-6"
 
         guide.linxw = Linx_par_name
         guide.loutxw = Loutx_par_name
@@ -418,6 +349,9 @@ class Elliptic(guide_elements.GuideElement):
         guide.W = self.W
         guide.enableGravity = self.enableGravity
 
+        end = self.current_instrument.add_component(self.end_component_name, "Arm")
+        end.set_AT([0, 0, self.get_length_name()], RELATIVE=guide)
+
 import numpy as np
 
 
@@ -498,4 +432,60 @@ def plot_element_vertical(ax, par_dict, color):
     ellipse_p_side, ellipse_n_side = ellipse_calculation(ellipse_pos, start, end, start_height, end_height, minor_axis)
 
     ax.plot(start + ellipse_pos, ellipse_p_side, color=color)
-    ax.plot(start + ellipse_pos, ellipse_n_side, color=color)
\ No newline at end of file
+    ax.plot(start + ellipse_pos, ellipse_n_side, color=color)
+
+def guide_width(par_dict, distance_unitless):
+    start = par_dict["start_point"]
+    end = par_dict["next_start_point"]
+    start_width = par_dict["start_width"]
+    end_width = par_dict["end_width"]
+    minor_axis = par_dict["minor_axis_x"]
+
+    ellipse_pos = (end-start)*distance_unitless
+    ellipse_p_side, ellipse_n_side = ellipse_calculation(ellipse_pos, start, end, start_width, end_width, minor_axis)
+    return 2.0 * ellipse_p_side
+
+def guide_height(par_dict, distance_unitless):
+    start = par_dict["start_point"]
+    end = par_dict["next_start_point"]
+    start_height = par_dict["start_height"]
+    end_height = par_dict["end_height"]
+    minor_axis = par_dict["minor_axis_y"]
+
+    ellipse_pos = (end-start)*distance_unitless
+    ellipse_p_side, ellipse_n_side = ellipse_calculation(ellipse_pos, start, end, start_height, end_height, minor_axis)
+    return 2.0 * ellipse_p_side
+
+def guide_dimensions(par_dict, distance_unitless, horizontal):
+    start = par_dict["start_point"]
+    end = par_dict["next_start_point"]
+
+    if horizontal:
+        start_dim = par_dict["start_width"]
+        end_dim = par_dict["end_width"]
+        minor_axis = par_dict["minor_axis_x"]
+    else:
+        start_dim = par_dict["start_height"]
+        end_dim = par_dict["end_height"]
+        minor_axis = par_dict["minor_axis_y"]
+
+    ellipse_pos = (end - start) * distance_unitless
+    ellipse_p_side, ellipse_n_side = ellipse_calculation(ellipse_pos, start, end, start_dim, end_dim, minor_axis)
+    return 2.0 * ellipse_p_side
+
+
+def center_line(start_pr, par_dict, distance_unit_less):
+
+    start = par_dict["start_point"]
+    end = par_dict["next_start_point"]
+
+    length = end - start
+    this_position = length*distance_unit_less
+
+    z_vector = np.array([0, 0, 1])
+    start_direction = start_pr.rotation
+    z_vector = start_direction.apply(z_vector)
+
+    updated_position = start_pr.position + this_position*z_vector
+
+    return PositionAndRotation(updated_position, start_pr.rotation)
\ No newline at end of file
