@@ -1,4 +1,6 @@
+import numpy as np
 from guide_bot.base_elements import guide_elements
+from guide_bot.base_elements.guide_elements import PositionAndRotation
 
 
 class Straight(guide_elements.GuideElement):
@@ -92,8 +94,8 @@ class Straight(guide_elements.GuideElement):
         added after the Origin component to ensure the correct order.
         """
 
-        guide = self.current_instrument.add_component(self.name, "Guide_gravity", after="Origin")
-        guide.set_AT([0, 0, self.start_point_parameter.name])
+        guide = self.current_instrument.add_component(self.name, "Guide_gravity")
+        guide.set_AT([0, 0, 0], RELATIVE=self.reference_component_name)
         
         guide.w1 = self.start_width.name
         guide.h1 = self.start_height.name
@@ -101,7 +103,7 @@ class Straight(guide_elements.GuideElement):
         guide.w2 = self.end_width.name
         guide.h2 = self.end_height.name
 
-        guide.l = self.get_length_name()
+        guide.l = self.get_length_name() + " - 1E-6"
 
         # Temporary reflectivity model
         guide.R0 = self.R0
@@ -110,6 +112,9 @@ class Straight(guide_elements.GuideElement):
         guide.alpha = self.alpha
         guide.W = self.W
 
+        end = self.current_instrument.add_component(self.end_component_name, "Arm")
+        end.set_AT([0, 0, self.get_length_name()], RELATIVE=guide)
+
 
 def plot_element_horizontal(ax, par_dict, color):
 
@@ -130,4 +135,52 @@ def plot_element_vertical(ax, par_dict, color):
     end_height = par_dict["end_height"]
 
     ax.plot([start, end], [0.5*start_height, 0.5*end_height], color=color)
-    ax.plot([start, end], [-0.5*start_height, -0.5*end_height], color=color)
\ No newline at end of file
+    ax.plot([start, end], [-0.5*start_height, -0.5*end_height], color=color)
+
+
+def guide_width(par_dict, distance_unit_less):
+    start_width = par_dict["start_width"]
+    end_width = par_dict["end_width"]
+
+    return start_width + (end_width - start_width) * distance_unit_less
+
+
+def guide_height(par_dict, distance_unit_less):
+    start_height = par_dict["start_height"]
+    end_height = par_dict["end_height"]
+
+    return start_height + (end_height - start_height)*distance_unit_less
+
+
+def guide_dimensions(par_dict, distance_unitless, horizontal):
+
+    if horizontal:
+        start_width = par_dict["start_width"]
+        end_width = par_dict["end_width"]
+
+        return start_width + (end_width - start_width) * distance_unitless
+
+    else:
+        start_height = par_dict["start_height"]
+        end_height = par_dict["end_height"]
+
+        return start_height + (end_height - start_height) * distance_unitless
+
+
+def center_line(start_pr, par_dict, distance_unit_less):
+
+    start = par_dict["start_point"]
+    end = par_dict["next_start_point"]
+
+    length = end - start
+    this_position = length*distance_unit_less
+
+    z_vector = np.array([0, 0, 1])
+
+    start_direction = start_pr.rotation
+    z_vector = start_direction.apply(z_vector)
+
+
+    updated_position = start_pr.position + this_position*z_vector
+
+    return PositionAndRotation(updated_position, start_pr.rotation)
\ No newline at end of file
