@@ -58,6 +58,7 @@ class Guide:
         self.constraints = []
 
         self.current_owner = None
+        self.required_components = set()
     
     def save_original(self):
         """
@@ -268,7 +269,24 @@ class Guide:
         # Moderator gets the first guide module to set up focusing
         moderator.add_to_instrument(instrument, instrument_parameters, guide)
 
-    def add_to_instrument(self, instrument, instrument_parameters, sample_dimensions):
+    def set_instrument_and_instr_parameters(self, instrument, instrument_parameters):
+        """
+        Sets McStasScript instrument and instrument parameter container
+
+        All elements are informed of the current instrument object and
+        instrument parameter container.
+
+        instrument : McStasScript instr object
+            Instrument object to which the guide should be added
+
+        instrument_parameters : InstrumentParameterContainer
+            The InstrumentParameterContainer with parameters and constraints
+        """
+
+        for element in self.guide_elements:
+            element.setup_instrument_and_parameters(instrument, instrument_parameters)
+
+    def add_to_instrument(self, sample_dimensions):
         """
         Adds McStasScript objects describing this guide to a instrument
 
@@ -283,39 +301,21 @@ class Guide:
         Parameters
         ----------
 
-        instrument : McStasScript instr object
-            Instrument object to which the guide should be added
-
-        instrument_parameters : InstrumentParameterContainer
-            The InstrumentParameterContainer with parameters and constraints
-
         sample_dimensions : list of length 2
             The width and height parameter for sample dimensions in a list
         """
 
-        """
-        sample.add_to_instrument(instrument, instrument_parameters)
-        sample.add_wavelength_parameters(instrument_parameters)
+        reference = "ABSOLUTE"
+        for element, next_element in zip(self.guide_elements[:-1], self.guide_elements[1:]):
+            element.reference_component_name = reference
+            reference = element.end_component_name
+            element.set_end_dimensions(next_element.start_width, next_element.start_height)
+            element.add_to_instr()
 
-        sample_width = ipars.FixedInstrumentParameter("sample_width", sample["width"])
-        sample_height = ipars.FixedInstrumentParameter("sample_height", sample["height"])
-
-        previous_start_dimensions = [sample_width, sample_height]
-        """
-
-        previous_start_dimensions = sample_dimensions
-
-        for guide in reversed(self.guide_elements):
-            guide.setup_instrument_and_parameters(instrument, instrument_parameters)
-            guide.set_end_dimensions(previous_start_dimensions[0], previous_start_dimensions[1])
-            guide.add_to_instr()
-
-            previous_start_dimensions = [guide.start_width, guide.start_height]
-
-        """
-        # Moderator gets the first guide module to set up focusing
-        moderator.add_to_instrument(instrument, instrument_parameters, guide)
-        """
+        last_element = self.guide_elements[-1]
+        last_element.reference_component_name = reference
+        last_element.set_end_dimensions(sample_dimensions[0], sample_dimensions[1])
+        last_element.add_to_instr()
 
     def write_log_file(self, filename):
         # Start file
@@ -325,6 +325,15 @@ class Guide:
             for element in self.guide_elements:
                 element.write_to_log(file)
 
+    def copy_components(self, destination):
+        """
+        Copies necessary components to destination
+        """
+        for element in self.guide_elements:
+            required_components = element.copy_components(destination)
+            for required_component in required_components:
+                self.required_components.add(required_component)
+
     def __repr__(self):
         """
         Provides a string describing the Guide object
@@ -541,7 +550,8 @@ class Project:
         # Save with dill
         package = {"scan_name": scan_name, "guide": guide, "settings": self.settings,
                    "sample": self.sample, "moderator": self.moderator,
-                   "analysis_moderators": self.analysis_moderators}
+                   "analysis_moderators": self.analysis_moderators,
+                   "required_components": guide.required_components}
         outfile = open(scan_name + ".plk", "wb")
         dill.dump(package, outfile)
         outfile.close()
@@ -586,6 +596,8 @@ class Project:
                 os.mkdir(guide_folder)
             except OSError:
                 raise RuntimeError("Could not create folder for guide!" + guide_folder)
+
+            guide.copy_components(guide_folder)
             os.chdir(guide_folder)
 
             scan.reset_configuration()
