@@ -1,6 +1,7 @@
 import copy
 import os
 import dill
+import shutil
 
 from guide_bot.parameters import instrument_parameters as ipars
 from guide_bot.parameters import instrument_parameter_container as ipars_container
@@ -35,6 +36,7 @@ class RunFromFile:
         self.moderator = package["moderator"]
         self.analysis_moderators = package["analysis_moderators"]
         self.sample = package["sample"]
+        self.required_components = package["required_components"]
 
         if settings is not None:
             # Allows easy update of any settings for cluster runs
@@ -169,6 +171,13 @@ class RunFromFile:
         except OSError:
             raise RuntimeError("Could not create folder for optimization data! " + foldername)
 
+        current_folder = os.getcwd()
+        for file_name in self.required_components:
+            origin = os.path.join(current_folder, file_name)
+            destination = os.path.join(current_folder, foldername, file_name)
+
+            shutil.copyfile(origin, destination)
+
     def create_instrument(self, moderator, instr_parameters, optimization_mode=True):
 
         self.guide.restore_original()
@@ -192,6 +201,22 @@ class RunFromFile:
         instrument = instr.McStas_instr(self.scan_name)
         instrument.add_component("Origin", "Progress_bar")
 
+        # Inform all guide elements of the instrument and instrument parameters
+        instr_parameters.set_current_category("guide")
+        self.guide.set_instrument_and_instr_parameters(instrument, instr_parameters)
+
+        # Before adding source, the sample needs to set wavelength range
+        instr_parameters.set_current_category("sample")
+        self.sample.add_sample_info(instr_parameters)
+
+        # Add guide system to the instrument
+        instr_parameters.set_current_category("guide")
+        self.guide.add_to_instrument(self.sample.get_size_parameters())
+
+        # Add source to the instrument
+        instr_parameters.set_current_category("moderator")
+        moderator.add_to_instrument(instrument, instr_parameters, self.guide.guide_elements[0])
+
         # Add sample to the instrument
         instr_parameters.set_current_category("sample")
         if optimization_mode:
@@ -200,19 +225,7 @@ class RunFromFile:
             self.sample.add_analysis_to_instrument(instrument, instr_parameters)
             self.sample.add_brilliance_analysis_to_instrument(instrument, instr_parameters)
 
-        self.sample.add_wavelength_parameters(instr_parameters)
 
-        sample_width = ipars.FixedInstrumentParameter("sample_width", self.sample["width"])
-        sample_height = ipars.FixedInstrumentParameter("sample_height", self.sample["height"])
-        sample_dimensions = [sample_width, sample_height]
-
-        # Add guide system to the instrument
-        instr_parameters.set_current_category("guide")
-        self.guide.add_to_instrument(instrument, instr_parameters, sample_dimensions)
-
-        # Add source to the instrument
-        instr_parameters.set_current_category("moderator")
-        moderator.add_to_instrument(instrument, instr_parameters, self.guide.guide_elements[0])
 
         instr_parameters.set_current_category(None) # Disable category system
         # Export the instrument parameters to the instrument to become parameters
