@@ -84,6 +84,9 @@ class InstrumentParameterContainer:
             # Parameters are only allowed once in the container
             return
 
+        if parameter.name in [par.name for par in self.all_parameters]:
+            raise RuntimeError("Two instrument parameters with the same name added! Name:" + parameter.name)
+
         if isinstance(parameter, RelativeFreeInstrumentParameter):
             self.relative_parameters.append(parameter)
         elif isinstance(parameter, FreeInstrumentParameter):
@@ -262,7 +265,6 @@ class InstrumentParameterContainer:
             if limit < 0:
                 raise RuntimeError("Was not able to set all parameters! Is there a circular dependency?")
 
-
     def extract_instrument_parameters(self, optimizer_x=None):
         """
         From optimizer state returns dictionary with parameter : value pairs
