@@ -144,6 +144,22 @@ class Parameters:
 
         return scan_dict
 
+    def get_fixed_dict(self):
+        """
+        Gets a dictionary of the parameters that are not scanned
+
+        Returns
+        -------
+        dictionary with parameters that are not scanned and their values
+
+        """
+        par_dict = {}
+        for par in self.parameters:
+            if par not in self.scanned_parameters:
+                par_dict[par] = self.parameters[par]
+
+        return par_dict
+
     def get_unit_dict(self, only_scanned=False):
         """
         Gets a dictionary of the units for the scanned parameters
@@ -195,7 +211,6 @@ class Parameters:
 
         return state_dict
 
-
     def __getitem__(self, name):
         """
         Providing [] operator that returns current value of requested parameter
